"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.factoriesFrom = exports.releaseAll = exports.prepareAll = exports.isReady = void 0;
var create_instance_1 = require("./create-instance");
var all_names_1 = require("./utils/all-names");
var assert_exists_1 = require("./utils/assert-exists");
var map_object_1 = require("./utils/map-object");
var narrow_object_1 = require("./utils/narrow-object");
var shallow_merge_1 = require("./utils/shallow-merge");
var $INSTANCES = Symbol('TRUE-DI::INSTANCES');
var itemsOf = function (container) {
    return (0, assert_exists_1.default)(container[$INSTANCES], 'Argument is not a container');
};
var isReady = function (container, name) {
    return itemsOf(container).has(name);
};
exports.isReady = isReady;
var prepareAll = function (container) {
    return (0, map_object_1.default)(container, function (name) { return container[name]; });
};
exports.prepareAll = prepareAll;
var releaseAll = function (container) {
    itemsOf(container).clear();
};
exports.releaseAll = releaseAll;
var factoriesFrom = function (container, names) {
    return (0, map_object_1.default)(container, function (name) { return function () { return container[name]; }; }, names);
};
exports.factoriesFrom = factoriesFrom;
var diContainer = function (privateFactories, publicFactories) {
    var _a;
    var instances = new Map();
    var factories = (publicFactories != null
        ? (0, shallow_merge_1.shallowMerge)(privateFactories, publicFactories)
        : privateFactories);
    var createInstance = (0, create_instance_1.default)(factories, instances);
    var container = (0, all_names_1.default)(factories).reduce(function (containerObj, name) {
        return Object.defineProperty(containerObj, name, {
            configurable: false,
            enumerable: Object.getOwnPropertyDescriptor(factories, name).enumerable,
            get: function () { return createInstance(container, name); },
            set: function (value) {
                if (value != null) {
                    throw new Error('Container does\'t allow to replace items');
                }
                instances.delete(name);
            },
        });
    }, Object.create(Object.create(null, (_a = {},
        _a[$INSTANCES] = {
            configurable: false, writable: false, enumerable: false, value: instances,
        },
        _a))));
    return publicFactories != null ? (0, narrow_object_1.default)(container, (0, all_names_1.default)(publicFactories)) : container;
};
exports.default = diContainer;
